@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Inspect musical modal functions
 
-our $VERSION = '0.0316';
+our $VERSION = '0.0400';
 
 use strictures 2;
 use AI::Prolog ();
@@ -25,6 +25,27 @@ has verbose => (
     default => sub { 0 },
 );
 
+has hash_results => (
+    is      => 'ro',
+    isa     => sub { croak "$_[0] is not a boolean" unless $_[0] =~ /^[01]$/ },
+    default => sub { 0 },
+);
+
+has _chord_key => (
+    is      => 'ro',
+    default => sub { [qw(method chord_note chord key_note key key_function key_roman)] },
+);
+
+has _pivot_chord_keys => (
+    is      => 'ro',
+    default => sub { [qw(method chord_note chord mode_note mode mode_function mode_roman key_note key key_function key_roman)] },
+);
+
+has _roman_key => (
+    is      => 'ro',
+    default => sub { [qw(method mode mode_roman key key_roman)] },
+);
+
 has _modes => (
     is => 'lazy',
 );
@@ -180,7 +201,7 @@ sub chord_key {
         defined $self->key          ? $self->key          : 'Key',
         defined $self->key_function ? $self->key_function : 'KeyFunction',
         defined $self->key_roman    ? $self->key_roman    : 'KeyRoman';
-    return $self->_querydb($query);
+    return $self->_querydb('chord_key', $query);
 }
 
 
@@ -197,7 +218,7 @@ sub pivot_chord_keys {
         defined $self->key           ? $self->key           : 'Key',
         defined $self->key_function  ? $self->key_function  : 'KeyFunction',
         defined $self->key_roman     ? $self->key_roman     : 'KeyRoman';
-    return $self->_querydb($query);
+    return $self->_querydb('pivot_chord_keys', $query);
 }
 
 
@@ -208,20 +229,29 @@ sub roman_key {
         defined $self->mode_roman ? $self->mode_roman : 'ModeRoman',
         defined $self->key        ? $self->key        : 'Key',
         defined $self->key_roman  ? $self->key_roman  : 'KeyRoman';
-    return $self->_querydb($query);
+    return $self->_querydb('roman_key', $query);
 }
 
 sub _querydb {
-    my ($self, $query) = @_;
+    my ($self, $method, $query) = @_;
 
-    warn "Query: $query\n" if $self->verbose;
+    warn "$method query: $query\n" if $self->verbose;
 
     $self->_prolog->query($query);
 
+    my $attr = '_' . $method;
+
     my @return;
 
     while (my $result = $self->_prolog->results) {
-        push @return, $result;
+        if ($self->hash_results) {
+            my %result;
+            @result{ @{ $self->$attr } } = @$result;
+            push @return, \%result;
+        }
+        else {
+            push @return, $result;
+        }
     }
 
     return \@return;
@@ -241,7 +271,7 @@ Music::ModalFunction - Inspect musical modal functions
 
 =head1 VERSION
 
-version 0.0316
+version 0.0400
 
 =head1 SYNOPSIS
 
@@ -263,16 +293,14 @@ version 0.0316
     chord_note   => 'g',
     chord        => 'maj',
     key_function => 'subdominant',
+    hash_results => 1,
   );
   $results = $m->pivot_chord_keys;
-  # [[ 'pivot_chord_keys', 'g', 'maj', 'c', 'ionian', 'dominant', 'r_V', 'd', 'dorian', 'subdominant', 'r_IV' ],
-  #  [ 'pivot_chord_keys', 'g', 'maj', 'c', 'ionian', 'dominant', 'r_V', 'd', 'ionian', 'subdominant', 'r_IV' ],
-  #  [ 'pivot_chord_keys', 'g', 'maj', 'c', 'ionian', 'dominant', 'r_V', 'd', 'mixolydian', 'subdominant', 'r_IV' ],
-  #  ...
-  #  [ 'pivot_chord_keys', 'g', 'maj', 'b', 'phrygian', 'submediant', 'r_VI', 'd', 'dorian', 'subdominant', 'r_IV' ],
-  #  [ 'pivot_chord_keys', 'g', 'maj', 'b', 'phrygian', 'submediant', 'r_VI', 'd', 'ionian', 'subdominant', 'r_IV' ],
-  #  [ 'pivot_chord_keys', 'g', 'maj', 'b', 'phrygian', 'submediant', 'r_VI', 'd', 'mixolydian', 'subdominant', 'r_IV' ]]
-  # Inspecting the result lists, we see that the answers are D Dorian, D Ionian, and D Mixolydian.
+  # [{ chord_note => 'g', chord => 'maj', mode_note => 'c', mode => 'ionian', mode_function => 'dominant', mode_roman => 'r_V', key_note => 'd', key => 'dorian', key_function => 'subdominant', key_roman => 'r_IV' },
+  #  { chord_note => 'g', chord => 'maj', mode_note => 'c', mode => 'ionian', mode_function => 'dominant', mode_roman => 'r_V', key_note => 'd', key => 'ionian', key_function => 'subdominant', key_roman => 'r_IV' },
+  #  { chord_note => 'g', chord => 'maj', mode_note => 'c', mode => 'ionian', mode_function => 'dominant', mode_roman => 'r_V', key_note => 'd', key => 'mixolydian', key_function => 'subdominant', key_roman => 'r_IV' },
+  #  ... ]
+  # Inspecting all the result lists, we see that the answers are D Dorian, D Ionian, and D Mixolydian.
 
 =head1 DESCRIPTION
 
